#ifndef _GPSPARSER
#define _GPSPARSER

#define GPS_PORT "/dev/ttyS0"
#define GPS_SPEED "9600"

#define NMEA_PORT "/dev/ttyS1"
#define NMEA_SPEED "4800"

#include <sys/types.h>		/* these 3 for open() */
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>		/* a buncha stuff */
#include <string.h>		/* strncmp() */
#include <termios.h>		/* tcgetattr() & Co. */
#include <stdio.h>		/* perror(), fprintf() */
#include <stdlib.h>		/* atoi(), atexit() */
#include <signal.h>		/* BSD semantics...do whatever your libc wants to get it */

/* Raw CMC format message structure */
struct rawmsgformat {
    unsigned char start;	/* Must be decimal 1 */
    unsigned char msgid;	/* Message ID# */
    unsigned char msgidcmpl;	/* Message ID# 1s' complement */
    unsigned char msglen;	/* Message length in bytes (not including header+checksum) */
    unsigned char message[255];	/* Message proper */
    unsigned char checksumlsb;	/* LSB of 16-bit checksum of message and header */
    unsigned char checksummsb;	/* MSB of checksum */
};

/* type conversion functions */
inline void raw2ARMdouble(unsigned char *, double *);
inline void ARMdouble2raw(unsigned char *, double *);
inline void ARMfloat2raw(unsigned char *, float *);
inline void raw2ARMfloat(unsigned char *, float *);
inline void int2raw(unsigned char *, int *);
inline void raw2int(unsigned char *, int *);
inline void short2raw(unsigned char *, short *);
inline void raw2short(unsigned char *, short *);

/* msg parsers */
int  setserialport (char *, char *);
int  setserialport2 (char *, char *);
int  main_loop (void);
int  process_user_input (void);
void parse_rx_msg (void);
void crapout (char *);

/* msg 20 params */

extern unsigned char hour;
extern unsigned char minute;
extern double seconds;
extern unsigned char day;
extern unsigned char month;
extern unsigned short year;
extern double latitude;
extern double longitude;
extern float altitude;
extern float clockbias;
extern float clockdrift;
extern float velocitynorth;
extern float velocityeast;
extern float velocityup;
extern float hfom;
extern float vfom;
extern float gpstime;
extern unsigned char navmode;
extern unsigned char numofsv;
extern unsigned char reserved;
extern unsigned short weekno;
extern char *NavModes[];

/* Outgoing CMC message protocols */
void cmc_tx_msg_63 (char, char *);
void cmc_tx_msg (char);

/* Misc stuff */
extern int serialfd;
extern int nmeafd;
void rxparser20(struct rawmsgformat msg);
void rxparser6(struct rawmsgformat msg);
void rxparser7(struct rawmsgformat msg);

#endif
