#include "gpsparser.h"

extern int serialfd;

/* Link Init */
void cmc_tx_msg_63 (char msgnum, char *passwd)
{
    unsigned char buffer[14];
    unsigned short checksum;
    int foo;

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 8;		/* msg length */
    strncpy (&buffer[4], passwd, 8);
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[12] = (checksum & 0x00FF);
    buffer[13] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}

/* Generic parser for short, no data messages */
void txparsershort(unsigned int msgnum)
{
    unsigned char buffer[6];
    unsigned short checksum;

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 0;		/* msg length */
    checksum = buffer[0] +
	buffer[1] +
	buffer[2] +
	buffer[3];
    buffer[04] = (checksum & 0x00FF);
    buffer[05] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}

void cmc_tx_msg (char msgnum) {
  txparsershort (msgnum);
}

/* Measurement block data request */
void txparser23(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[7];
    unsigned short checksum;

    unsigned int txrate;

    fp = fopen("./23.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 23.cmc");
	return;
    }
    foo = fscanf(fp, "%d%*c", &txrate);
    if (foo != 1) {
	fprintf(stderr, "Mangled file 23.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 23.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 1;		/* msg length */
    buffer[4] = txrate;
    checksum = buffer[0] +
	buffer[1] +
	buffer[2] +
	buffer[3] +
	buffer[4];
    buffer[05] = (checksum & 0x00FF);
    buffer[06] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}


/* Initiated BIT request */
void txparser51(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[7];
    unsigned short checksum;

    unsigned int BIT;

    fp = fopen("./51.cmc", "r");
    if (fp == NULL) {
	perror("Missing file 51.cmc");
	return;
    }
    foo = fscanf(fp, "%d%*c", &BIT);
    if (foo != 1) {
	fprintf(stderr, "Mangled file 51.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 51.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 1;		/* msg length */
    buffer[4] = BIT;
    checksum = buffer[0] +
	buffer[1] +
	buffer[2] +
	buffer[3] +
	buffer[4];
    buffer[05] = (checksum & 0x00FF);
    buffer[06] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}


/* Set Channel deselection */
void txparser64(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[16];
    unsigned short checksum;

    unsigned int mask1, mask2;

    fp = fopen("./64.cmc", "r");
    if (fp == NULL) {
	perror("Missing file 64.cmc");
	return;
    }
    foo = fscanf(fp, "%c%c%c%c%c%c%c%c%*c%d%*c%d%*c",
		 &buffer[4],
		 &buffer[5],
		 &buffer[6],
		 &buffer[7],
		 &buffer[8],
		 &buffer[9],
		 &buffer[10],
		 &buffer[11],
		 &mask1,
		 &mask2);
    if (foo != 10) {
	fprintf(stderr, "Mangled file 64.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 64.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 10;		/* msg length */
    buffer[12] = mask1;
    buffer[13] = mask2;
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[14] = (checksum & 0x00FF);
    buffer[15] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}

/* Set Operating Mode */
void txparser80(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[38];
    unsigned short checksum;

    double double1, double2, double3;

    fp = fopen("./80.cmc", "r");
    if (fp == NULL) {
	perror("Missing file 80.cmc");
	return;
    }
    foo = fscanf(fp, "%c%c%c%c%c%c%c%c%*c%lf%*c%lf%*c%lf%*c",
		 &buffer[4],
		 &buffer[5],
		 &buffer[6],
		 &buffer[7],
		 &buffer[8],
		 &buffer[9],
		 &buffer[10],
		 &buffer[11],
		 &double1,
		 &double2,
		 &double3);
    if (foo != 11) {
	fprintf(stderr, "Mangled file 80.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 80.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 32;		/* msg length */
    ARMdouble2raw(&buffer[12], &double1);
    ARMdouble2raw(&buffer[20], &double2);
    ARMdouble2raw(&buffer[28], &double3);
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[36] = (checksum & 0x00FF);
    buffer[37] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}

/* Set Mask Angle */
void txparser81(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[18];
    unsigned short checksum;

    float maskangle;

    fp = fopen("./81.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 81.cmc");
	return;
    }
    foo = fscanf(fp, "%f%*c%c%c%c%c%c%c%c%c%*c",
		 &maskangle,
		 &buffer[8],
		 &buffer[9],
		 &buffer[10],
		 &buffer[11],
		 &buffer[12],
		 &buffer[13],
		 &buffer[14],
		 &buffer[15]);
    if (foo != 9) {
	fprintf(stderr, "Mangled file 81.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 81.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 12;		/* msg length */
    ARMfloat2raw(&buffer[4], &maskangle);
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[16] = (checksum & 0x00FF);
    buffer[17] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}

/* Select/Define Datum */
#if 0
void txparser88(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[38];
    unsigned short checksum;

    unsigned int function;
    unsigned int datumnumber;
    signed short deltax;
    signed short deltay;
    signed short deltaz;
    double semimajor;

    fp = fopen("./88.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 88.cmc");
	return;
    }
    foo = fscanf(fp, "%d%*c%d%*c%d%*c%d%*c%d%*c%lf%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c",
		 &function,
		 &datumnumber,
		 &deltax,
		 &deltay,
		 &deltaz,
		 &semimajor,
		 &buffer[20],
		 &buffer[21],
		 &buffer[22],
		 &buffer[23],
		 &buffer[24],
		 &buffer[25],
		 &buffer[26],
		 &buffer[27],
		 &buffer[28],
		 &buffer[29],
		 &buffer[30],
		 &buffer[31],
		 &buffer[32],
		 &buffer[33],
		 &buffer[34],
		 &buffer[35]);
    if (foo != 22) {
	fprintf(stderr, "Mangled file 88.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 88.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 32;		/* msg length */
    buffer[4] = function;
    buffer[5] = datumnumber;
    short2raw(&buffer[6], &deltax);
    short2raw(&buffer[8], &deltay);
    short2raw(&buffer[10], &deltaz);
    ARMdouble2raw(&buffer[12], &semimajor);
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[36] = (checksum & 0x00FF);
    buffer[37] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}
#endif

/* Set SV Deselect */
void txparser90(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[18];
    unsigned short checksum;

    unsigned int bmp1;
    unsigned int bmp2;
    unsigned int bmp3;
    unsigned int bmp4;

    fp = fopen("./90.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 90.cmc");
	return;
    }
    foo = fscanf(fp, "%c%c%c%c%c%c%c%c%*c%d%*c%d%*c%d%*c%d%*c",
		 &buffer[4],
		 &buffer[5],
		 &buffer[6],
		 &buffer[7],
		 &buffer[8],
		 &buffer[9],
		 &buffer[10],
		 &buffer[11],
		 &bmp1,
		 &bmp2,
		 &bmp3,
		 &bmp4);
    if (foo != 12) {
	fprintf(stderr, "Mangled file 90.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 90.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 12;		/* msg length */
    buffer[12] = bmp1;
    buffer[13] = bmp2;
    buffer[14] = bmp3;
    buffer[15] = bmp4;
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[16] = (checksum & 0x00FF);
    buffer[17] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}

/* Differential Message Configuration */
void txparser91(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[8];
    unsigned short checksum;

    unsigned int msgtypeproto;
    unsigned int msgrate;

    fp = fopen("./91.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 91.cmc");
	return;
    }
    foo = fscanf(fp, "%d%*c%d%*c",
		 &msgtypeproto,
		 &msgrate);
    if (foo != 2) {
	fprintf(stderr, "Mangled file 91.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 91.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 2;		/* msg length */
    buffer[4] = msgtypeproto;
    buffer[5] = msgrate;
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[6] = (checksum & 0x00FF);
    buffer[7] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}


/* FIXME This spec is too broken to ever work */
/* Set Date/Time & GPS Time alignment mode */
#if 0
void txparser103(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[24];
    unsigned short checksum;

    unsigned int validtime1;
    unsigned int validtime2;
    unsigned int hours;
    unsigned int minutes;
    unsigned int seconds;
    unsigned int day;
    unsigned int month;
    unsigned int year;

    fp = fopen("./103.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 103.cmc");
	return;
    }
    foo = fscanf(fp, "%d%*c%d%*c%c%c%c%c%c%c%c%c%*c%d%*c%d%*c%d%*c%d%*c%d%*c%d%*c",
		 &validtime1,
		 &validtime2,
		 &buffer[6],	/* spec is broken here */
		 &buffer[7],
		 &buffer[8],
		 &buffer[9],
		 &buffer[10],
		 &buffer[11],
		 &buffer[12],
		 &hours,
		 &minutes,
		 &seconds,
		 &day,
		 &month,
		 &year,
		 &buffer[19]);	/* spec is broken here */
    if (foo != 16) {
	fprintf(stderr, "Mangled file 103.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 91.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 18;		/* msg length *//*This is wrong */
    buffer[4] = validtime1;
    buffer[5] = validtime2;
    buffer[13] = hours;
    buffer[14] = minutes;
    buffer[15] = seconds;
    buffer[16] = day;
    buffer[17] = month;
    buffer[18] = year;
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[20] = (checksum & 0x00FF);
    buffer[21] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}
#endif

/* Set Default CMC Binary Message list */
void txparser105(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[30];
    unsigned short checksum;

    unsigned int mask1;
    unsigned int mask2;
    unsigned int mask3;
    unsigned int mask4;

    fp = fopen("./105.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 105.cmc");
	return;
    }
    foo = fscanf(fp, "%d%*c%d%*c%d%*c%d%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c%c%*c",
		 &mask1,
		 &mask2,
		 &mask3,
		 &mask4,
		 &buffer[20],
		 &buffer[21],
		 &buffer[22],
		 &buffer[23],
		 &buffer[24],
		 &buffer[25],
		 &buffer[26],
		 &buffer[27]);
    if (foo != 12) {
	fprintf(stderr, "Mangled file 105.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 105.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 12;		/* msg length */
    int2raw(&buffer[4], &mask1);
    int2raw(&buffer[8], &mask2);
    int2raw(&buffer[12], &mask3);
    int2raw(&buffer[16], &mask4);
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[28] = (checksum & 0x00FF);
    buffer[29] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}


/* Configure Main Port Mode */
void txparser110(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[7];
    unsigned short checksum;

    unsigned int config;

    fp = fopen("./105.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 105.cmc");
	return;
    }
    foo = fscanf(fp, "%d%*c",
		 &config);
    if (foo != 1) {
	fprintf(stderr, "Mangled file 105.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 105.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 1;		/* msg length */
    buffer[4] = config;
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[5] = (checksum & 0x00FF);
    buffer[6] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}


/* Switch to reprogramming mode */
void txparser112(unsigned int msgnum)
{
    FILE *fp;
    int foo;
    unsigned char buffer[7];
    unsigned short checksum;

    unsigned int config;

    fp = fopen("./112.cmc", "r");
    if (fp == NULL) {
	perror("Cannot open file 112.cmc");
	return;
    }
    foo = fscanf(fp, "%d%*c",
		 &config);
    if (foo != 1) {
	fprintf(stderr, "Mangled file 112.cmc\n");
	return;
    }
    foo = fclose(fp);
    if (foo == EOF)
	perror("Error closing file 112.cmc");

    buffer[0] = 1;
    buffer[1] = msgnum;
    buffer[2] = ~buffer[1];
    buffer[3] = 1;		/* msg length */
    buffer[4] = config;
    for (foo = 0, checksum = 0; foo < (buffer[3] + 4); foo++) {
	checksum += buffer[foo];
    }
    buffer[5] = (checksum & 0x00FF);
    buffer[6] = (checksum & 0xFF00) >> 8;
    write(serialfd, buffer, sizeof(buffer));
}
